﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Taschenrechner.operators;

namespace Taschenrechner.states
{
    class Context
    {
        public readonly EvalState evalState = new EvalState();
        public readonly NormalState normalState = new NormalState();
        public readonly NumberState numberState = new NumberState();

        public readonly Dictionary<char, IOperator> operatorDic;
        private Dictionary<string, IState> stateDic;

        private Register register_;
        public Register register
        {
            get { return register_; }
        }

        private Stack stack_;
        public Stack stack
        {
            get { return stack_; }
        }

        private Buffer buffer_;
        public Buffer buffer
        {
            get { return buffer_; }
        }

        public IState currentState
        {
            get { 
                return stateDic[register[Constants.REGISTER_STATE]];
            }
            set { 
                register[Constants.REGISTER_STATE] = value.GetCode();
            }
        }

        public Context()
        {
            register_ = new Register();
            stack_ = new Stack(register);
            buffer_ = new Buffer(register);

            operatorDic = new Dictionary<char, IOperator>();
            operatorDic.Add('+', new PlusOperator());
            operatorDic.Add('-', new MinusOperator());
            operatorDic.Add('*', new MultiplyOperator());
            operatorDic.Add('/', new DivOperator());
            operatorDic.Add('%', new ModOperator());
            operatorDic.Add('~', new ChangeSignOperator());
            operatorDic.Add('!', new BitwiseNegationOperator());
            operatorDic.Add('&', new AndOperator());
            operatorDic.Add('=', new EqualsOperator());
            operatorDic.Add('>', new GreaterOperator());
            operatorDic.Add('<', new SmallerOperator());
            operatorDic.Add('|', new OrOperator());
            operatorDic.Add('r', new ReadStackOperator());
            operatorDic.Add('s', new WriteStackOperator());
            operatorDic.Add('a', new ApplyOperator());

            stateDic = new Dictionary<string, IState>();
            stateDic.Add(normalState.GetCode(), normalState);
            stateDic.Add(numberState.GetCode(), numberState);
            stateDic.Add(evalState.GetCode(), evalState);

            currentState = normalState;
        }

        public void HandleNumber(char number)
        {
            currentState.HandleNumber(this, number);
        }

        public void HandleDelimiter()
        {
            currentState.HandleDelimiter(this);
        }

        public void HandleQuote()
        {
            currentState.HandleQuote(this);
        }

        public void HandleOperator(char op)
        {
            currentState.HandleOperator(this, op);
        }
    }
}
